@@ -28,6 +28,22 @@ const COLORREF crefInFocus = (RGB(222, 255, 222));  // Light green
 const COLORREF crefNoFocus = (RGB(255, 255, 255));  // White
 const COLORREF crefBlack   = (RGB(  0,   0,   0));  // Black
 
+// timer event numbers used to by ControlExtns for ListBox tooltips. See DboxMain.h
+#define TIMER_LB_HOVER     0x0A
+#define TIMER_LB_SHOWING   0x0B 
+
+/*
+HOVER_TIME_LB       The length of time the pointer must remain stationary
+                    within a tool's bounding rectangle before the tool tip
+                    window appears.
+
+TIMEINT_LB_SHOWING The length of time the tool tip window remains visible
+                   if the pointer is stationary within a tool's bounding
+                   rectangle.
+*/
+#define HOVER_TIME_LB      1000
+#define TIMEINT_LB_SHOWING 5000
+
 /////////////////////////////////////////////////////////////////////////////
 // CStaticExtn
 
@@ -473,7 +489,7 @@ bool CListBoxExtn::ShowToolTip(int nItem, const bool bVisible)
   CPoint pt;
   ::GetCursorPos(&pt);
 
-  pt.y += ::GetSystemMetrics(SM_CYCURSOR); // height of cursor
+  pt.y += ::GetSystemMetrics(SM_CYCURSOR) / 2; // half-height of cursor
 
   m_pLBToolTips->SetWindowPos(&wndTopMost, pt.x, pt.y, 0, 0,
                               SWP_NOSIZE | SWP_NOACTIVATE | SWP_SHOWWINDOW);
@@ -809,6 +825,7 @@ void CSecEditExtn::OnSecureUpdate()
 BEGIN_MESSAGE_MAP(CSymbolEdit, CEdit)
   //{{AFX_MSG_MAP(CSymbolEdit)
   ON_WM_CHAR()
+  ON_MESSAGE(WM_PASTE, OnPaste)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -830,3 +847,40 @@ void CSymbolEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
       CEdit::OnChar(nChar, 0, nFlags);
   }
 }
+
+LRESULT CSymbolEdit::OnPaste(WPARAM , LPARAM )
+{
+  // Only allow symbols to be pasted and stop duplicates
+  if (!OpenClipboard() || !IsClipboardFormatAvailable(CF_UNICODETEXT))
+    return 0L; 
+ 
+  std::wstring cs_data;
+	HANDLE hData = GetClipboardData(CF_UNICODETEXT);
+  if (hData != NULL) {
+    wchar_t *buffer = (wchar_t *)GlobalLock(hData);
+    if (buffer != NULL) {
+      cs_data = buffer;
+      GlobalUnlock(hData);
+    }
+  }
+	CloseClipboard();
+
+  CString cs_text, cs_oldtext;;
+  GetWindowText(cs_text);
+  cs_oldtext = cs_text;
+
+  // Must not be alphanumeric
+  for (size_t i = 0; i < cs_data.length(); i++) {
+    wchar_t wChar = cs_data.at(i);
+    if ((_istalpha(wChar) == 0 && _istdigit(wChar) == 0)) {
+      // Must not have duplicates
+      if (cs_text.Find(wChar) == -1)
+        cs_text += wChar;
+    }
+  }
+
+  if (cs_text.Compare(cs_oldtext) != 0)
+    SetWindowText(cs_text);
+
+  return 0L;
+}
\ No newline at end of file
