@@ -26,6 +26,7 @@
 #include "PWFileDialog.h"
 #include "DisplayFSBkupFiles.h"
 #include "ExpPWListDlg.h"
+#include "AddEdit_DateTimes.h"
 
 #include "WZPropertySheet.h"
 
@@ -52,6 +53,7 @@
 #include <vector>
 
 using namespace std;
+using pws_os::CUUID;
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -204,11 +206,13 @@ BOOL DboxMain::OpenOnInit()
 
   bool go_ahead = false;
   /*
-   * If BAD_DIGEST or LIMIT_REACHED,
+   * If file's corrupted, read error or LIMIT_REACHED (demo),
    * the we prompt the user, and continue or not per user's input.
    * A bit too subtle for switch/case on rc2...
    */
-  if (rc2 == PWScore::BAD_DIGEST) {
+  if (rc2 == PWScore::BAD_DIGEST ||
+      rc2 == PWScore::TRUNCATED_FILE ||
+      rc2 == PWScore:: READ_FAIL) {
     CGeneralMsgBox gmb;
     CString cs_title(MAKEINTRESOURCE(IDS_FILEREADERROR)), cs_msg;
     cs_msg.Format(IDS_FILECORRUPT, m_core.GetCurFile().c_str());
@@ -217,7 +221,7 @@ BOOL DboxMain::OpenOnInit()
       goto exit;
     }
     go_ahead = true;
-  } // BAD_DIGEST
+  } // read error
 #ifdef DEMO
   if (rc2 == PWScore::LIMIT_REACHED) {
     CGeneralMsgBox gmb;
@@ -337,8 +341,10 @@ int DboxMain::New()
   m_DDNotes.SetStaticState(false);
   m_DDURL.SetStaticState(false);
   m_DDemail.SetStaticState(false);
+  m_DDAutotype.SetStaticState(false);
 
   UpdateMenuAndToolBar(true);
+  UpdateStatusBar();
 
   // Set timer for user-defined idle lockout, if selected (DB preference)
   KillTimer(TIMER_LOCKDBONIDLETIMEOUT);
@@ -402,9 +408,8 @@ int DboxMain::NewFile(StringX &newfilename)
       return PWScore::USER_CANCEL;
   }
 
-  CPasskeySetup dbox_pksetup(this);
-  //app.m_pMainWnd = &dbox_pksetup;
-  rc = dbox_pksetup.DoModal();
+  CPasskeySetup pksetup(this);
+  rc = pksetup.DoModal();
 
   if (rc == IDCANCEL)
     return PWScore::USER_CANCEL;  //User cancelled password entry
@@ -413,7 +418,7 @@ int DboxMain::NewFile(StringX &newfilename)
   m_core.ReInit(true);
 
   ClearData();
-  PWSprefs::GetInstance()->SetDatabasePrefsToDefaults();
+
   const StringX &oldfilename = m_core.GetCurFile();
   // The only way we're the locker is if it's locked & we're !readonly
   if (!oldfilename.empty() &&
@@ -428,7 +433,7 @@ int DboxMain::NewFile(StringX &newfilename)
   m_core.LockFile(newfilename.c_str(), locker);
 
   m_core.SetReadOnly(false); // new file can't be read-only...
-  m_core.NewFile(dbox_pksetup.m_passkey);
+  m_core.NewFile(pksetup.GetPassKey());
   m_bDBNeedsReading = false;
   
   // Tidy up filters
@@ -458,8 +463,11 @@ int DboxMain::Close(const bool bTrySave)
     if (m_bOpen) {
       // try and save it first
       int rc = SaveIfChanged();
-      if (rc != PWScore::SUCCESS)
+      if (rc != PWScore::SUCCESS && rc != PWScore::USER_DECLINED_SAVE)
         return rc;
+
+      // Reset changed flag to stop being asked again (only if rc == PWScore::USER_DECLINED_SAVE)
+      SetChanged(Clear);
     }
   }
 
@@ -477,10 +485,10 @@ int DboxMain::Close(const bool bTrySave)
   ClearData();
 
   // Zero entry UUID selected and first visible at minimize and group text
-  memset(m_LUUIDSelectedAtMinimize, 0, sizeof(uuid_array_t));
-  memset(m_TUUIDSelectedAtMinimize, 0, sizeof(uuid_array_t));
-  memset(m_LUUIDVisibleAtMinimize, 0, sizeof(uuid_array_t));
-  memset(m_TUUIDVisibleAtMinimize, 0, sizeof(uuid_array_t));
+  m_LUUIDSelectedAtMinimize = CUUID::NullUUID();
+  m_TUUIDSelectedAtMinimize = CUUID::NullUUID();
+  m_LUUIDVisibleAtMinimize = CUUID::NullUUID();
+  m_TUUIDVisibleAtMinimize = CUUID::NullUUID();
   m_sxSelectedGroup.clear();
   m_sxVisibleGroup.clear();
 
@@ -502,6 +510,7 @@ int DboxMain::Close(const bool bTrySave)
   m_DDNotes.SetStaticState(false);
   m_DDURL.SetStaticState(false);
   m_DDemail.SetStaticState(false);
+  m_DDAutotype.SetStaticState(false);
 
   app.SetTooltipText(L"PasswordSafe");
   UpdateSystemTray(CLOSED);
@@ -666,9 +675,12 @@ int DboxMain::Open(const StringX &sx_Filename, const bool bReadOnly,  const bool
   }
 
   rc = SaveIfChanged();
-  if (rc != PWScore::SUCCESS)
+  if (rc != PWScore::SUCCESS && rc != PWScore::USER_DECLINED_SAVE)
     return rc;
 
+  // Reset changed flag to stop being asked again (only if rc == PWScore::USER_DECLINED_SAVE)
+  SetChanged(Clear);
+
   // If we were using a different file, unlock it do this before 
   // GetAndCheckPassword() as that routine gets a lock on the new file
   if (!m_core.GetCurFile().empty()) {
@@ -724,10 +736,10 @@ int DboxMain::Open(const StringX &sx_Filename, const bool bReadOnly,  const bool
   ClearData();
 
   // Zero entry UUID selected and first visible at minimize and group text
-  memset(m_LUUIDSelectedAtMinimize, 0, sizeof(uuid_array_t));
-  memset(m_TUUIDSelectedAtMinimize, 0, sizeof(uuid_array_t));
-  memset(m_LUUIDVisibleAtMinimize, 0, sizeof(uuid_array_t));
-  memset(m_TUUIDVisibleAtMinimize, 0, sizeof(uuid_array_t));
+  m_LUUIDSelectedAtMinimize = CUUID::NullUUID();
+  m_TUUIDSelectedAtMinimize = CUUID::NullUUID();
+  m_LUUIDVisibleAtMinimize = CUUID::NullUUID();
+  m_TUUIDVisibleAtMinimize = CUUID::NullUUID();
   m_sxSelectedGroup.clear();
   m_sxVisibleGroup.clear();
 
@@ -847,10 +859,7 @@ void DboxMain::PostOpenProcessing()
 
   UUIDList RUElist;
   m_core.GetRUEList(RUElist);
-  for (UUIDListRIter riter = RUElist.rbegin();
-             riter != RUElist.rend(); riter++) {
-    m_RUEList.AddRUEntry(riter->uuid);
-  }
+  m_RUEList.SetRUEList(RUElist);
 
   // Set timer for user-defined idle lockout, if selected (DB preference)
   KillTimer(TIMER_LOCKDBONIDLETIMEOUT);
@@ -965,7 +974,7 @@ int DboxMain::CheckEmergencyBackupFiles(StringX sx_Filename, StringX &passkey)
   // Close original - don't save anything
   Close(false);
 
-  // Now open the one selected by the user in R/O mode
+  // Now open the one selected by the user in R-O mode
   sx_fullfilename = vValidEBackupfiles[-dsprc].dbp.database;
   rc = m_core.ReadFile(sx_fullfilename, passkey);
   ASSERT(rc == PWScore::SUCCESS);
@@ -1157,9 +1166,8 @@ int DboxMain::SaveIfChanged()
         else
           return PWScore::CANT_OPEN_FILE;
       case IDNO:
-        // Reset changed flag to stop being asked again
-        SetChanged(Clear);
-        break;
+        // It is a success but we need to know that the user said no!
+        return PWScore::USER_DECLINED_SAVE;
     }
   }
   return PWScore::SUCCESS;
@@ -1253,8 +1261,7 @@ int DboxMain::SaveAs()
   }
 
   // Save file UUID, clear it to generate new one, restore if necessary
-  uuid_array_t file_uuid_array;
-  m_core.GetFileUUID(file_uuid_array);
+  pws_os::CUUID file_uuid = m_core.GetFileUUID();
   m_core.ClearFileUUID();
 
   UUIDList RUElist;
@@ -1266,7 +1273,7 @@ int DboxMain::SaveAs()
   m_core.ClearChangedNodes();
 
   if (rc != PWScore::SUCCESS) {
-    m_core.SetFileUUID(file_uuid_array);
+    m_core.SetFileUUID(file_uuid); // restore uuid after failed save-as
     m_core.UnlockFile2(newfile.c_str());
     DisplayFileWriteError(rc, newfile);
     return PWScore::CANT_OPEN_FILE;
@@ -1384,9 +1391,7 @@ void DboxMain::OnExportText()
   sx_temp = m_core.GetCurFile();
   if (sx_temp.empty()) {
     //  Database has not been saved - prompt user to do so first!
-    cs_temp.LoadString(IDS_SAVEBEFOREEXPORT);
-    cs_text.Format(IDS_SAVEBEFOREPROCESS, L"", cs_temp);
-    gmb.AfxMessageBox(cs_text);
+    gmb.AfxMessageBox(IDS_SAVEBEFOREPROCESS);
     return;
   }
 
@@ -1475,9 +1480,7 @@ void DboxMain::OnExportXML()
   sx_temp = m_core.GetCurFile();
   if (sx_temp.empty()) {
     //  Database has not been saved - prompt user to do so first!
-    cs_temp.LoadString(IDS_SAVEBEFOREEXPORT);
-    cs_text.Format(IDS_SAVEBEFOREPROCESS, L"", cs_temp);
-    gmb.AfxMessageBox(cs_text);
+    gmb.AfxMessageBox(IDS_SAVEBEFOREPROCESS);
     return;
   }
 
@@ -1539,7 +1542,7 @@ int DboxMain::DoExportXML(const StringX &sx_Filename, const bool bAll,
 
   ReportAdvancedOptions(prpt, bAdvanced, bAll ? WZAdvanced::EXPORT_XML : WZAdvanced::EXPORT_ENTRYXML);
 
-  // do the export
+  // Do the export
   int rc = m_core.WriteXMLFile(sx_Filename, bsFields, subgroup_name,
                                subgroup_object, subgroup_function,
                                delimiter, numExported, &orderedItemList,
@@ -1621,7 +1624,7 @@ void DboxMain::OnImportText()
     bool bWasEmpty = m_core.GetNumEntries() == 0;
     std::wstring strError;
     StringX TxtFileName = fd.GetPathName();
-    int numImported(0), numSkipped(0), numPWHErrors(0), numRenamed(0);
+    int numImported(0), numSkipped(0), numPWHErrors(0), numRenamed(0), numWarnings(0);
     wchar_t delimiter = dlg.m_defimpdelim[0];
     bool bImportPSWDsOnly = dlg.m_bImportPSWDsOnly == TRUE;
 
@@ -1663,8 +1666,15 @@ void DboxMain::OnImportText()
         // deliberate fallthru
       default:
       {
-        if (pcmd != NULL)
+        if (pcmd != NULL) {
           Execute(pcmd);
+          const size_t n = ((MultiCommands *)pcmd)->GetSize();
+          for (size_t i = 0; i < n; i++) {
+            int iw;
+            if (((MultiCommands *)pcmd)->GetRC(i, iw))
+              numWarnings += iw;
+          }
+        }
 
         rpt.WriteLine();
         CString cs_type;
@@ -1697,6 +1707,11 @@ void DboxMain::OnImportText()
           cs_temp += cs_tmp;
         }
 
+        if (numWarnings != 0) {
+          CString cs_tmp(MAKEINTRESOURCE(IDS_WITHWARNINGS));
+          cs_temp += cs_tmp;
+        }
+
         cs_title.LoadString(rc == PWScore::SUCCESS ? IDS_COMPLETE : IDS_OKWITHERRORS);
 
         ChangeOkUpdate();
@@ -1722,13 +1737,14 @@ void DboxMain::OnImportText()
   }
 }
 
-void DboxMain::OnImportKeePass()
+void DboxMain::OnImportKeePassV1CSV()
 {
   if (m_core.IsReadOnly()) // disable in read-only mode
     return;
 
-  CString cs_text, cs_title, cs_temp;
-  cs_text.LoadString(IDS_PICKKEEPASSFILE);
+  UINT uiReasonCode(0);
+  CString cs_title, cs_msg;
+  cs_title.LoadString(IDS_PICKKEEPASSFILE);
   std::wstring dir;
   if (m_core.GetCurFile().empty())
     dir = PWSdirs::GetSafeDir();
@@ -1739,13 +1755,13 @@ void DboxMain::OnImportKeePass()
   }
 
   CPWFileDialog fd(TRUE,
-                   L"txt",
+                   L"csv",
                    NULL,
                    OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES,
-                   CString(MAKEINTRESOURCE(IDS_FDF_T_C_ALL)),
+                   CString(MAKEINTRESOURCE(IDS_FDF_CSV_ALL)),
                    this);
 
-  fd.m_ofn.lpstrTitle = cs_text;
+  fd.m_ofn.lpstrTitle = cs_title;
 
   if (!dir.empty())
     fd.m_ofn.lpstrInitialDir = dir.c_str();
@@ -1759,25 +1775,43 @@ void DboxMain::OnImportKeePass()
     PostQuitMessage(0);
     return;
   }
+
   if (rc == IDOK) {
     CGeneralMsgBox gmb;
     bool bWasEmpty = m_core.GetNumEntries() == 0;
     Command *pcmd = NULL;
     StringX KPsFileName = fd.GetPathName();
-    rc = m_core.ImportKeePassTextFile(KPsFileName, pcmd);
+    int numImported, numSkipped, numRenamed;
+
+    // Create report as we go
+    CReport rpt;
+    std::wstring str_text;
+    LoadAString(str_text, IDS_RPTIMPORTKPV1CSV);
+    rpt.StartReport(str_text.c_str(), m_core.GetCurFile().c_str());
+    LoadAString(str_text, IDS_TEXT);
+    cs_msg.Format(IDS_IMPORTFILE, str_text.c_str(), KPsFileName.c_str());
+    rpt.WriteLine((LPCWSTR)cs_msg);
+    rpt.WriteLine();
+
+    rc = m_core.ImportKeePassV1CSVFile(KPsFileName, numImported, numSkipped, numRenamed,
+                                       uiReasonCode, rpt, pcmd);
     switch (rc) {
       case PWScore::CANT_OPEN_FILE:
       {
-        cs_temp.Format(IDS_CANTOPENREADING, KPsFileName.c_str());
+        cs_msg.Format(IDS_CANTOPENREADING, KPsFileName.c_str());
         cs_title.LoadString(IDS_FILEOPENERROR);
-        gmb.MessageBox(cs_temp, cs_title, MB_OK | MB_ICONWARNING);
+        delete [] pcmd;
         break;
       }
       case PWScore::INVALID_FORMAT:
+      case PWScore::FAILURE:
       {
-        cs_temp.Format(IDS_INVALIDFORMAT, KPsFileName.c_str());
-        cs_title.LoadString(IDS_FILEREADERROR);
-        gmb.MessageBox(cs_temp, cs_title, MB_OK | MB_ICONWARNING);
+        if (uiReasonCode > 0)
+          cs_msg.LoadString(uiReasonCode);
+        else
+          cs_msg.Format(IDS_INVALIDFORMAT, KPsFileName.c_str());
+        cs_title.LoadString(IDS_IMPORTFAILED);
+        delete [] pcmd;
         break;
       }
       case PWScore::SUCCESS:
@@ -1789,8 +1823,137 @@ void DboxMain::OnImportKeePass()
         // May need to update menu/toolbar if original database was empty
         if (bWasEmpty)
           UpdateMenuAndToolBar(m_bOpen);
+
+        rpt.WriteLine();
+        CString cs_type;
+        cs_type.LoadString(numImported == 1 ? IDSC_ENTRY : IDSC_ENTRIES);
+        cs_msg.Format(IDS_RECORDSIMPORTED, numImported, cs_type);
+        rpt.WriteLine((LPCWSTR)cs_msg);
+
+        cs_title.LoadString(rc == PWScore::SUCCESS ? IDS_COMPLETE : IDS_OKWITHERRORS);
         break;
     } // switch
+    rpt.EndReport();
+
+    gmb.SetTitle(cs_title);
+    gmb.SetMsg(cs_msg);
+    gmb.SetStandardIcon(rc == PWScore::SUCCESS ? MB_ICONINFORMATION : MB_ICONEXCLAMATION);
+    gmb.AddButton(IDS_OK, IDS_OK, TRUE, TRUE);
+    gmb.AddButton(IDS_VIEWREPORT, IDS_VIEWREPORT);
+    INT_PTR rc = gmb.DoModal();
+    if (rc == IDS_VIEWREPORT)
+      ViewReport(rpt);
+  }
+}
+
+void DboxMain::OnImportKeePassV1TXT()
+{
+  if (m_core.IsReadOnly()) // disable in read-only mode
+    return;
+
+  UINT uiReasonCode(0);
+  CString cs_title, cs_msg;
+  cs_title.LoadString(IDS_PICKKEEPASSFILE);
+  std::wstring dir;
+  if (m_core.GetCurFile().empty())
+    dir = PWSdirs::GetSafeDir();
+  else {
+    std::wstring cdrive, cdir, dontCare;
+    pws_os::splitpath(m_core.GetCurFile().c_str(), cdrive, cdir, dontCare, dontCare);
+    dir = cdrive + cdir;
+  }
+
+  CPWFileDialog fd(TRUE,
+                   L"txt",
+                   NULL,
+                   OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES,
+                   CString(MAKEINTRESOURCE(IDS_FDF_TXT_ALL)),
+                   this);
+
+  fd.m_ofn.lpstrTitle = cs_title;
+
+  if (!dir.empty())
+    fd.m_ofn.lpstrInitialDir = dir.c_str();
+
+  INT_PTR rc = fd.DoModal();
+
+  if (m_inExit) {
+    // If U3ExitNow called while in CPWFileDialog,
+    // PostQuitMessage makes us return here instead
+    // of exiting the app. Try resignalling 
+    PostQuitMessage(0);
+    return;
+  }
+
+  if (rc == IDOK) {
+    CGeneralMsgBox gmb;
+    bool bWasEmpty = m_core.GetNumEntries() == 0;
+    Command *pcmd = NULL;
+    StringX KPsFileName = fd.GetPathName();
+    int numImported, numSkipped, numRenamed;
+
+    // Create report as we go
+    CReport rpt;
+    std::wstring str_text;
+    LoadAString(str_text, IDS_RPTIMPORTKPV1TXT);
+    rpt.StartReport(str_text.c_str(), m_core.GetCurFile().c_str());
+    LoadAString(str_text, IDS_TEXT);
+    cs_msg.Format(IDS_IMPORTFILE, str_text.c_str(), KPsFileName.c_str());
+    rpt.WriteLine((LPCWSTR)cs_msg);
+    rpt.WriteLine();
+
+    rc = m_core.ImportKeePassV1TXTFile(KPsFileName, numImported, numSkipped, numRenamed,
+                                       uiReasonCode, rpt, pcmd);
+    switch (rc) {
+      case PWScore::CANT_OPEN_FILE:
+      {
+        cs_msg.Format(IDS_CANTOPENREADING, KPsFileName.c_str());
+        cs_title.LoadString(IDS_FILEOPENERROR);
+        delete [] pcmd;
+        break;
+      }
+      case PWScore::INVALID_FORMAT:
+      {
+        if (uiReasonCode > 0)
+          cs_msg.LoadString(uiReasonCode);
+        else
+          cs_msg.Format(IDS_INVALIDFORMAT, KPsFileName.c_str());
+        cs_title.LoadString(IDS_IMPORTFAILED);
+        delete [] pcmd;
+        break;
+      }
+      case PWScore::SUCCESS:
+      default: // deliberate fallthru
+        if (pcmd != NULL)
+          Execute(pcmd);
+
+        RefreshViews();
+        ChangeOkUpdate();
+        // May need to update menu/toolbar if original database was empty
+        if (bWasEmpty)
+          UpdateMenuAndToolBar(m_bOpen);
+
+        rpt.WriteLine();
+        CString cs_type;
+        cs_type.LoadString(numImported == 1 ? IDSC_ENTRY : IDSC_ENTRIES);
+        cs_msg.Format(IDS_RECORDSIMPORTED, numImported, cs_type);
+        rpt.WriteLine((LPCWSTR)cs_msg);
+
+        cs_title.LoadString(rc == PWScore::SUCCESS ? IDS_COMPLETE : IDS_OKWITHERRORS);
+
+        break;
+    } // switch
+    // Finish Report
+    rpt.EndReport();
+
+    gmb.SetTitle(cs_title);
+    gmb.SetMsg(cs_msg);
+    gmb.SetStandardIcon(rc == PWScore::SUCCESS ? MB_ICONINFORMATION : MB_ICONEXCLAMATION);
+    gmb.AddButton(IDS_OK, IDS_OK, TRUE, TRUE);
+    gmb.AddButton(IDS_VIEWREPORT, IDS_VIEWREPORT);
+    INT_PTR rc = gmb.DoModal();
+    if (rc == IDS_VIEWREPORT)
+      ViewReport(rpt);
   }
 }
 
@@ -1816,7 +1979,6 @@ void DboxMain::OnImportXML()
   const std::wstring XSDfn(L"pwsafe.xsd");
   std::wstring XSDFilename = PWSdirs::GetXMLDir() + XSDfn;
 
-#if USE_XML_LIBRARY == MSXML || USE_XML_LIBRARY == XERCES
   if (!pws_os::FileExists(XSDFilename)) {
     CGeneralMsgBox gmb;
     cs_temp.Format(IDSC_MISSINGXSD, XSDfn.c_str());
@@ -1824,7 +1986,6 @@ void DboxMain::OnImportXML()
     gmb.MessageBox(cs_temp, cs_title, MB_OK | MB_ICONSTOP);
     return;
   }
-#endif
 
   CImportXMLDlg dlg;
   INT_PTR status = dlg.DoModal();
@@ -1869,7 +2030,6 @@ void DboxMain::OnImportXML()
     std::wstring strXMLErrors, strSkippedList, strPWHErrorList, strRenameList;
     CString XMLFilename = fd.GetPathName();
     int numValidated, numImported, numSkipped, numRenamed, numPWHErrors;
-    bool bBadUnknownFileFields, bBadUnknownRecordFields;
     bool bImportPSWDsOnly = dlg.m_bImportPSWDsOnly == TRUE;
 
     CWaitCursor waitCursor;  // This may take a while!
@@ -1890,7 +2050,6 @@ void DboxMain::OnImportXML()
                               XSDFilename.c_str(), bImportPSWDsOnly,
                               strXMLErrors, strSkippedList, strPWHErrorList, strRenameList,
                               numValidated, numImported, numSkipped, numPWHErrors, numRenamed,
-                              bBadUnknownFileFields, bBadUnknownRecordFields,
                               rpt, pcmd);
     waitCursor.Restore();  // Restore normal cursor
 
@@ -1913,23 +2072,10 @@ void DboxMain::OnImportXML()
           Execute(pcmd);
 
         if (!strXMLErrors.empty() ||
-            bBadUnknownFileFields || bBadUnknownRecordFields ||
             numRenamed > 0 || numPWHErrors > 0) {
           if (!strXMLErrors.empty())
             csErrors = strXMLErrors + L"\n";
 
-          if (bBadUnknownFileFields) {
-            CString cs_type(MAKEINTRESOURCE(IDS_HEADER));
-            cs_temp.Format(IDS_XMLUNKNFLDIGNORED, cs_type);
-            csErrors += cs_temp + L"\n";
-          }
-
-          if (bBadUnknownRecordFields) {
-            CString cs_type(MAKEINTRESOURCE(IDS_RECORD));
-            cs_temp.Format(IDS_XMLUNKNFLDIGNORED, cs_type);
-            csErrors += cs_temp + L"\n";
-          }
-
           if (!csErrors.empty()) {
             rpt.WriteLine(csErrors.c_str());
           }
@@ -2008,6 +2154,106 @@ void DboxMain::OnProperties()
   dlg.DoModal();
 }
 
+void DboxMain::OnChangeMode()	 
+{
+  // From StatusBar and menu
+  const bool bWasRO = IsDBReadOnly();
+
+  if (!bWasRO) {
+    // Try to save if any changes done to database
+    int rc = SaveIfChanged();
+    if (rc != PWScore::SUCCESS && rc != PWScore::USER_DECLINED_SAVE)
+      return;
+
+    if (rc == PWScore::USER_DECLINED_SAVE) {
+	     // But ask just in case	 
+       CGeneralMsgBox gmb;	 
+       CString cs_msg(MAKEINTRESOURCE(IDS_BACKOUT_CHANGES)), cs_title(MAKEINTRESOURCE(IDS_CHANGEMODE));	 
+       INT_PTR rc = gmb.MessageBox(cs_msg, cs_title, MB_YESNO | MB_ICONQUESTION);	 
+ 	 
+       if (rc == IDNO)	 
+         return;
+
+      // User said No to the save - so we must back-out all changes since last save
+      while (m_core.IsChanged()) {
+        OnUndo();
+      }
+    }
+ 
+    // Reset changed flag to stop being asked again (only if rc == PWScore::USER_DECLINED_SAVE)
+    SetChanged(Clear);
+
+    // Clear the Commands
+    m_core.ClearCommands();
+  }
+
+  CGeneralMsgBox gmb;
+  CString cs_msg, cs_title(MAKEINTRESOURCE(IDS_CHANGEMODE_FAILED));
+  std::wstring locker = L"";
+  int iErrorCode;
+  bool brc = m_core.ChangeMode(locker, iErrorCode);
+  if (brc) {
+    UpdateStatusBar();
+  } else {
+    // Better give them the bad news!
+    bool bInUse = false;
+    UINT uiMsg = 0;
+    if (bWasRO) {
+      switch (iErrorCode) {
+        case PWScore::DB_HAS_CHANGED:
+          // We did get the lock but the DB has been changed
+          // Note: PWScore has already freed the lock
+          // The user must close and re-open it in R/W mode
+          uiMsg = IDS_CM_FAIL_REASON3;
+          break;
+        
+        case PWScore::CANT_GET_LOCK:
+        {
+          CString cs_user_and_host, cs_PID;
+          cs_user_and_host = (CString)locker.c_str();
+          int i_pid = cs_user_and_host.ReverseFind(L':');
+          if (i_pid > -1) {
+            // If PID present then it is ":%08d" = 9 chars in length
+            ASSERT((cs_user_and_host.GetLength() - i_pid) == 9);
+            cs_PID.Format(IDS_PROCESSID, cs_user_and_host.Right(8));
+            cs_user_and_host = cs_user_and_host.Left(i_pid);
+          } else {
+            cs_PID = L"";
+          }
+
+          cs_msg.Format(IDS_CM_FAIL_REASON1, cs_user_and_host, cs_PID);
+          bInUse = true;
+          break;
+        }
+        case PWSfile::CANT_OPEN_FILE:
+          uiMsg = IDS_CM_FAIL_REASON4;
+          break;
+        case PWSfile::END_OF_FILE:
+          uiMsg = IDS_CM_FAIL_REASON5;
+          break;
+        default:
+          ASSERT(0);
+      }
+    } else {
+      // Don't need fail code when going from R/W to R-O - only one issue -
+      // could not release the lock!
+      uiMsg = IDS_CM_FAIL_REASON2;
+    }
+
+    if (bInUse) {
+      // Big message
+      gmb.SetTitle(cs_title);
+      gmb.SetMsg(cs_msg);
+      gmb.SetStandardIcon(MB_ICONWARNING);
+      gmb.AddButton(IDS_CLOSE, IDS_CLOSE);
+      gmb.DoModal();
+    } else {
+      cs_msg.LoadString(uiMsg);
+      gmb.MessageBox(cs_msg, cs_title, MB_OK | MB_ICONWARNING);
+    }
+  }
+}
+
 void DboxMain::OnCompare()
 {
   if (m_core.GetCurFile().empty() || m_core.GetNumEntries() == 0) {
@@ -2406,16 +2652,16 @@ LRESULT DboxMain::OnProcessCompareResultFunction(WPARAM wParam, LPARAM lFunction
   PWScore *pcore;
   st_CompareInfo *st_info;
   LRESULT lres(FALSE);
-  uuid_array_t entryUUID;
+  CUUID entryUUID;
 
   st_info = (st_CompareInfo *)wParam;
 
   if (st_info->clicked_column == CCompareResultsDlg::CURRENT) {
     pcore = st_info->pcore0;
-    memcpy(entryUUID, st_info->uuid0, sizeof(entryUUID));
+    entryUUID = st_info->uuid0;
   } else {
     pcore = st_info->pcore1;
-    memcpy(entryUUID, st_info->uuid1, sizeof(entryUUID));
+    entryUUID = st_info->uuid1;
   }
 
   switch ((int)lFunction) {
@@ -2443,13 +2689,13 @@ LRESULT DboxMain::OnProcessCompareResultFunction(WPARAM wParam, LPARAM lFunction
   return lres;
 }
 
-LRESULT DboxMain::ViewCompareResult(PWScore *pcore, uuid_array_t &entryUUID)
+LRESULT DboxMain::ViewCompareResult(PWScore *pcore, const CUUID &entryUUID)
 {  
   ItemListIter pos = pcore->Find(entryUUID);
   ASSERT(pos != pcore->GetEntryEndIter());
   CItemData *pci = &pos->second;
 
-  // View the correct entry and make sure R/O
+  // View the correct entry and make sure R-O
   bool bSaveRO = pcore->IsReadOnly();
   pcore->SetReadOnly(true);
 
@@ -2460,7 +2706,7 @@ LRESULT DboxMain::ViewCompareResult(PWScore *pcore, uuid_array_t &entryUUID)
   return FALSE;
 }
 
-LRESULT DboxMain::EditCompareResult(PWScore *pcore, uuid_array_t &entryUUID)
+LRESULT DboxMain::EditCompareResult(PWScore *pcore, const CUUID &entryUUID)
 {
   ItemListIter pos = pcore->Find(entryUUID);
   ASSERT(pos != pcore->GetEntryEndIter());
@@ -2471,7 +2717,7 @@ LRESULT DboxMain::EditCompareResult(PWScore *pcore, uuid_array_t &entryUUID)
 }
 
 LRESULT DboxMain::CopyCompareResult(PWScore *pfromcore, PWScore *ptocore,
-                                    uuid_array_t &fromUUID, uuid_array_t &toUUID)
+                                    const CUUID &fromUUID, const CUUID &toUUID)
 {
   bool bWasEmpty = ptocore->GetNumEntries() == 0;
 
@@ -2524,7 +2770,7 @@ LRESULT DboxMain::CopyCompareResult(PWScore *pfromcore, PWScore *ptocore,
 }
 
 LRESULT DboxMain::SynchCompareResult(PWScore *pfromcore, PWScore *ptocore,
-                                     uuid_array_t &fromUUID, uuid_array_t &toUUID)
+                                     const CUUID &fromUUID, const CUUID &toUUID)
 {
   // Synch 1 entry *pfromcore -> *ptocore
   CItemData::FieldBits bsFields;
@@ -2975,11 +3221,11 @@ void DboxMain::ReportAdvancedOptions(CReport *pRpt, const bool bAdvanced, const
     int ifields[] = {CItemData::PASSWORD, CItemData::NOTES, CItemData::URL,
                      CItemData::AUTOTYPE, CItemData::PWHIST, CItemData::POLICY,
                      CItemData::RUNCMD, CItemData::DCA, CItemData::EMAIL,
-                     CItemData::SYMBOLS};
+                     CItemData::PROTECTED, CItemData::SYMBOLS};
     UINT uimsgids[] = {IDS_COMPPASSWORD, IDS_COMPNOTES, IDS_COMPURL,
                        IDS_COMPAUTOTYPE, IDS_COMPPWHISTORY, IDS_COMPPWPOLICY,
                        IDS_COMPRUNCOMMAND, IDS_COMPDCA, IDS_COMPEMAIL,
-                       IDS_COMPSYMBOLS};
+                       IDS_COMPPROTECTED, IDS_COMPSYMBOLS};
     ASSERT(_countof(ifields) == _countof(uimsgids));
 
     // Time fields
