@@ -592,7 +592,7 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     SetUpMenuStrings(pPopupMenu);
 
   // We have done for all except the Edit and View menus
-  if (uiMenuID != ID_EDITMENU && uiMenuID != ID_VIEWMENU)
+  if (uiMenuID != ID_FILEMENU && uiMenuID != ID_EDITMENU && uiMenuID != ID_VIEWMENU)
     return;
 
   if (bItemSelected) {
@@ -600,6 +600,14 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     ASSERT(pci != NULL);
   }
 
+	// Change the 'Change Mode' text as appropriate	 
+  if (uiMenuID == ID_FILEMENU) {	 
+    pPopupMenu->ModifyMenu(ID_MENUITEM_CHANGEMODE, MF_BYCOMMAND,	 
+                           ID_MENUITEM_CHANGEMODE,	 
+                           bReadOnly ? CS_READWRITE : CS_READONLY);	 
+    return;	 
+  }
+
   // If View menu selected (contains 'Flattened &List' menu item)
   if (uiMenuID == ID_VIEWMENU) {
     // Delete Show Find Toolbar menu item - don't know if there or previously deleted
@@ -628,6 +636,9 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     pPopupMenu->CheckMenuItem(ID_MENUITEM_SHOW_ALL_EXPIRY, MF_BYCOMMAND |
                               m_bExpireDisplayed ? MF_CHECKED : MF_UNCHECKED);
 
+    pPopupMenu->EnableMenuItem(ID_FILTERMENU, MF_BYCOMMAND |
+             (m_bUnsavedDisplayed || m_bExpireDisplayed) ? MF_GRAYED : MF_ENABLED);
+
     pPopupMenu->ModifyMenu(ID_MENUITEM_APPLYFILTER, MF_BYCOMMAND |
                            m_bFilterActive ? MF_CHECKED : MF_UNCHECKED,
                            ID_MENUITEM_APPLYFILTER,
@@ -636,6 +647,10 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     pPopupMenu->CheckMenuRadioItem(ID_MENUITEM_NEW_TOOLBAR,
                                    ID_MENUITEM_OLD_TOOLBAR,
                                    m_toolbarMode, MF_BYCOMMAND);
+
+    // Disable View Reports menu if no database open as we need the full path
+    // of the database to get to its reports!
+    pPopupMenu->EnableMenuItem(ID_REPORTSMENU, MF_BYCOMMAND | (m_bOpen ? MF_ENABLED : MF_GRAYED));
     goto exit;
   }  // View menu
 
@@ -973,7 +988,10 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
       break;
   }
 
-  if (bDoShortcuts || minfo.dwMenuData == ID_EDITMENU || minfo.dwMenuData == ID_VIEWMENU)
+  if (bDoShortcuts ||
+      minfo.dwMenuData == ID_FILEMENU ||
+      minfo.dwMenuData == ID_EDITMENU ||
+      minfo.dwMenuData == ID_VIEWMENU)
     CustomiseMenu(pPopupMenu, (UINT)minfo.dwMenuData, bDoShortcuts);
 
   static int iLangPos = -1;
